/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: PersistenceXmlFileAnalyzer.java 369 2006-04-21 08:18:35Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.objectweb.easybeans.persistence.xml;

import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.jar.JarFile;
import java.util.zip.ZipEntry;

import javax.persistence.spi.PersistenceProvider;

import org.objectweb.easybeans.log.JLog;
import org.objectweb.easybeans.log.JLogFactory;
import org.objectweb.easybeans.persistence.PersistenceUnitManager;

/**
 * Class used to analyze a given archive (by analyzing the persistence.xml file
 * if any).
 * @author Florent Benoit
 */
public final class PersistenceXmlFileAnalyzer {

    /**
     * Directory where persistence.xml file should be.
     */
    private static final String DIRECTORY_PERSISTENCE_XML_FILE = "META-INF";

    /**
     * Name of the persistence.xml file.
     */
    private static final String PERSISTENCE_XML_FILE = "persistence.xml";

    /**
     * Logger.
     */
    private static JLog logger = JLogFactory.getLog(PersistenceXmlFileAnalyzer.class);

    /**
     * Utility class, no public constructor.
     */
    private PersistenceXmlFileAnalyzer() {

    }

    /**
     * Detects and analyze the META-INF/persistence.xml file.
     * @param archive the file to analyze (or directory) in order to find a
     *        persistence.xml file.
     * @param classLoader the classloader used to load the persistence provider
     *        class.
     * @return a persistence unit manager (which can manage the peristence
     *         contexts).
     * @throws PersistenceXmlFileAnalyzerException if detection or analyze
     *         fails.
     */
    public static PersistenceUnitManager analyzePersistenceXmlFile(final File archive, final ClassLoader classLoader)
            throws PersistenceXmlFileAnalyzerException {
        URL persistenceXmlURL = null;
        // if archive is a file, needs to look into the jar file
        if (archive.isFile()) {

            // Try to see if the archive contains the persistence.xml file.
            JarFile jarFile = null;
            try {
                jarFile = new JarFile(archive);
            } catch (IOException e) {
                throw new PersistenceXmlFileAnalyzerException("File '" + jarFile + "' is not a valid jar file.", e);
            }

            // Gets the entry.
            String persistenceEntry = DIRECTORY_PERSISTENCE_XML_FILE + '/' + PERSISTENCE_XML_FILE;
            ZipEntry zipEntry = jarFile.getEntry(persistenceEntry);
            if (zipEntry != null) {
                String jarUrl = "jar:file:" + archive.getPath() + "!/" + persistenceEntry;
                try {
                    persistenceXmlURL = new URL(jarUrl);
                } catch (MalformedURLException e) {
                    throw new PersistenceXmlFileAnalyzerException("Error while trying to build an URL with '" + jarUrl
                            + "'", e);
                }
            }
            try {
                jarFile.close();
            } catch (IOException e) {
                logger.error("Problem while closing jar file '" + jarFile + "'.");
            }

        } else {
            // directory mode
            File persistenceXmlFile = new File(archive, DIRECTORY_PERSISTENCE_XML_FILE + File.separator
                    + PERSISTENCE_XML_FILE);
            if (persistenceXmlFile.exists()) {
                try {
                    persistenceXmlURL = persistenceXmlFile.toURL();
                } catch (MalformedURLException e) {
                    throw new PersistenceXmlFileAnalyzerException("Cannot get URL from file '" + persistenceXmlFile
                            + "'.");
                }
            }
        }

        // Now, do the parsing and fill the structure.
        JPersistenceUnitInfo[] persistenceUnitInfos = null;
        if (persistenceXmlURL != null) {
            try {
                persistenceUnitInfos = JPersistenceUnitInfoHelper.getPersistenceUnitInfo(persistenceXmlURL);
            } catch (JPersistenceUnitInfoException e) {
                throw new PersistenceXmlFileAnalyzerException("Cannot parse the URL '" + persistenceXmlURL + "'.", e);
            }

            for (JPersistenceUnitInfo persistenceUnitInfo : persistenceUnitInfos) {
                try {
                    // sets the archive
                    persistenceUnitInfo.addJarFile(archive.toURL());

                    // Set the root url
                    persistenceUnitInfo.setPersistenceUnitRootUrl(archive.toURL());
                } catch (MalformedURLException e) {
                    throw new PersistenceXmlFileAnalyzerException("Cannot get the URL on the jar file '" + archive
                            + "'.", e);
                }

                // sets the classloader
                persistenceUnitInfo.setClassLoader(classLoader);

                // instantiate persistence provider
                Class persistenceProviderClass;
                try {
                    ClassLoader ctxLoader = Thread.currentThread().getContextClassLoader();
                    persistenceProviderClass = ctxLoader.loadClass(persistenceUnitInfo
                            .getPersistenceProviderClassName());
                } catch (ClassNotFoundException e) {
                    throw new PersistenceXmlFileAnalyzerException("Cannot load the persistence provider class '"
                            + persistenceUnitInfo.getPersistenceProviderClassName() + "'.");
                }
                PersistenceProvider persistenceProvider;
                try {
                    persistenceProvider = (PersistenceProvider) persistenceProviderClass.newInstance();
                } catch (InstantiationException e) {
                    throw new PersistenceXmlFileAnalyzerException("Cannot instantiate the persistence provider class '"
                            + persistenceUnitInfo.getPersistenceProviderClassName() + "'.", e);
                } catch (IllegalAccessException e) {
                    throw new PersistenceXmlFileAnalyzerException("Cannot instantiate the persistence provider class '"
                            + persistenceUnitInfo.getPersistenceProviderClassName() + "'.", e);
                }

                // Set persistence provider
                persistenceUnitInfo.setPersistenceProvider(persistenceProvider);

            }

            // create persistence unit manager
            return new PersistenceUnitManager(persistenceUnitInfos);
        }
        // nothing found, return nothing(null)
        return null;
    }

}
