/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: ClientRPCInvocationHandler.java 548 2006-05-30 17:06:53Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.objectweb.easybeans.rpc;

import java.lang.reflect.Method;
import java.rmi.RemoteException;
import java.rmi.ServerException;
import java.util.HashMap;

import javax.ejb.EJBException;

import org.objectweb.easybeans.rpc.api.ClientRPC;
import org.objectweb.easybeans.rpc.api.EJBRequest;
import org.objectweb.easybeans.rpc.api.EJBResponse;
import org.objectweb.easybeans.rpc.api.RPCException;
import org.objectweb.easybeans.rpc.util.Hash;



/**
 * This class sends an EJB request to the server and send back to the client the
 * response.
 * @author Florent Benoit
 */
public class ClientRPCInvocationHandler extends AbsInvocationHandler {

    /**
     * Id for serializable class.
     */
    private static final long serialVersionUID = 1852625501781836250L;


    /**
     * Build a new Invocation handler.
     * @param containerId the id of the container that will be called on the remote side.
     * @param factoryName the name of the remote factory.
     */
    public ClientRPCInvocationHandler(final String containerId, final String factoryName) {
        super(containerId, factoryName);
    }

    /**
     * Processes a method invocation on a proxy instance and returns the result.
     * This method will be invoked on an invocation handler when a method is
     * invoked on a proxy instance that it is associated with.
     * @param proxy the proxy instance that the method was invoked on
     * @param method the <code>Method</code> instance corresponding to the
     *        interface method invoked on the proxy instance. The declaring
     *        class of the <code>Method</code> object will be the interface
     *        that the method was declared in, which may be a superinterface of
     *        the proxy interface that the proxy class inherits the method
     *        through.
     * @param args an array of objects containing the values of the arguments
     *        passed in the method invocation on the proxy instance, or
     *        <code>null</code> if interface method takes no arguments.
     *        Arguments of primitive types are wrapped in instances of the
     *        appropriate primitive wrapper class, such as
     *        <code>java.lang.Integer</code> or <code>java.lang.Boolean</code>.
     * @return the value to return from the method invocation on the proxy
     *         instance.
     * @throws Exception the exception to throw from the method invocation on
     *         the proxy instance.
     */
    public Object invoke(final Object proxy, final Method method, final Object[] args) throws Exception {
        // Methods on the Object.class are not send on the remote side
        if (method.getDeclaringClass().getName().equals("java.lang.Object")) {
            return handleObjectMethods(method, args);
        }

        ClientRPC client = RPC.getClient();

        if (getHashedMethods() == null) {
            setHashedMethods(new HashMap<Method, Long>());
        }

        Long hashLong = getHashedMethods().get(method);
        if (hashLong == null) {
            hashLong = Long.valueOf(Hash.hashMethod(method));
            getHashedMethods().put(method, hashLong);
        }

        long hash = hashLong.longValue();

        EJBRequest request = new JEJBRequest(method.getName(), hash, args, getContainerId(), getFactoryName(), getBeanId());

        // send response
        EJBResponse response;
        try {
            response = client.sendEJBRequest(request);
        } catch (RPCException e) {
            Throwable originalException = e;
            // Try to extract exception.
            Throwable serverException = e.getCause();
            if (serverException != null && serverException instanceof ServerException) {
                Throwable remoteException = serverException.getCause();
                if (remoteException != null && remoteException instanceof RemoteException) {
                    if (originalException.getCause() != null) {
                        originalException = remoteException.getCause();
                    } else {
                        originalException = remoteException;
                    }
                }
            }

            // Handle the answer
            Class[] exceptions = method.getExceptionTypes();
            if (exceptions != null) {
                for (Class clazz : exceptions) {
                    if (clazz.isInstance(originalException) && originalException instanceof Exception) {
                        throw (Exception) originalException;
                    }
                }
            }
            if (originalException instanceof Exception) {
                throw new EJBException((Exception) originalException.getCause());
            }
            throw new RuntimeException(originalException);
        }
        setBeanId(response.getBeanId());
        return response.getValue();
    }

}
