/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: Embedded.java 678 2006-06-18 18:20:06Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.objectweb.easybeans.server;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;
import java.net.MalformedURLException;
import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.hsqldb.Server;
import org.objectweb.carol.jndi.ns.NameServiceException;
import org.objectweb.carol.jndi.ns.NameServiceManager;
import org.objectweb.carol.util.configuration.ConfigurationException;
import org.objectweb.carol.util.configuration.ConfigurationRepository;
import org.objectweb.easybeans.api.EZBContainer;
import org.objectweb.easybeans.api.EZBContainerConfig;
import org.objectweb.easybeans.api.EZBServer;
import org.objectweb.easybeans.container.JContainer3;
import org.objectweb.easybeans.container.JContainerConfig;
import org.objectweb.easybeans.deployer.Deployer;
import org.objectweb.easybeans.ds.DummyDataSource;
import org.objectweb.easybeans.jms.joram.JoramException;
import org.objectweb.easybeans.jms.joram.JoramService;
import org.objectweb.easybeans.jmx.CommonsModelerException;
import org.objectweb.easybeans.jmx.CommonsModelerHelper;
import org.objectweb.easybeans.jmx.JMXRemoteException;
import org.objectweb.easybeans.jmx.JMXRemoteHelper;
import org.objectweb.easybeans.jmx.MBeanServerException;
import org.objectweb.easybeans.jmx.MBeanServerHelper;
import org.objectweb.easybeans.jmx.MBeansException;
import org.objectweb.easybeans.jmx.MBeansHelper;
import org.objectweb.easybeans.log.JLog;
import org.objectweb.easybeans.log.JLogFactory;
import org.objectweb.easybeans.rpc.rmi.server.RMIServerRPC;
import org.objectweb.easybeans.rpc.rmi.server.RMIServerRPCImpl;
import org.objectweb.easybeans.transaction.JTransactionManager;

/**
 * Allows to create an embedded EJB3 server.
 * @author Florent Benoit
 */
public class Embedded implements EZBServer {

    /**
     * Default sleep value (for server loop).
     */
    private static final int SLEEP_VALUE = 10000;

    /**
     * Logger.
     */
    private static JLog logger = JLogFactory.getLog(Embedded.class);

    /**
     * Internal (global) counter of all embedded instance created.
     */
    private static int counter = 0;

    /**
     * ID of this embedded server.
     */
    private Integer id = null;

    /**
     * Configuration of this server.
     */
    private ServerConfig config = null;

    /**
     * Map of managed ejb3 containers.
     */
    private Map<String, EZBContainer> containers = null;

    /**
     * List of Callbacks factories.
     */
    private List<EasyBeansConfigurationExtension> extensionFactories = null;


    /**
     * Link to the joram service (if started).
     */
    private JoramService joramService = null;

    /**
     * Creates a new Embedded server.<br>
     * It will take default values of configuration.
     */
    public Embedded() {
        this(new ServerConfig());
    }

    /**
     * Creates a new embedded server with a given configuration.
     * @param config the server configuration.
     */
    public Embedded(final ServerConfig config) {
        this.id = new Integer(counter++);
        this.config = config;
        this.containers = new HashMap<String, EZBContainer>();
        this.extensionFactories = new ArrayList<EasyBeansConfigurationExtension>();
        // register to the list
        EmbeddedManager.addEmbedded(this);

        // configure
        configure();

    }

    /**
     * Starts the EJB3 server.
     * @throws EmbeddedException if there is a failure while starting the
     *         server.
     */
    public void start() throws EmbeddedException {
        long tStart = System.currentTimeMillis();

        // Use ExtensionFactories ?
        if (!config.getExtensionFactories().isEmpty()) {
            List<String> classnames = config.getExtensionFactories();
            for (String fqn : classnames) {
                EasyBeansConfigurationExtension factory = null;
                try {
                    factory = Class.forName(fqn).asSubclass(EasyBeansConfigurationExtension.class).newInstance();
                } catch (Exception e) {
                    throw new EmbeddedException("Cannot instantiate CallbackFacory : " + fqn, e);
                }
                this.extensionFactories.add(factory);
            }
        }

        // start the registry ?
        if (config.startRegistry()) {
            startRegistry();
        }

        if (config.isUsingNaming()) {
            // url.pkg for java:
            System.setProperty("java.naming.factory.url.pkgs", "org.objectweb.easybeans.naming.pkg");
        }

        // set the deployer used for this component.
        Deployer.setEmbedded(this);

        if (config.isUsingMBeans()) {

            // init Modeler Registry
            try {
                CommonsModelerHelper.initRegistry();
            } catch (CommonsModelerException e) {
                throw new EmbeddedException("Cannot init MBean server", e);
            }

            // Start MBeanServer (if any)
            try {
                MBeanServerHelper.startMBeanServer();
            } catch (MBeanServerException e) {
                throw new EmbeddedException("Cannot start MBean server", e);
            }

            try {
                JMXRemoteHelper.startConnector();
            } catch (JMXRemoteException e) {
                throw new EmbeddedException("Cannot start JMX Remote connector", e);
            }

            // register the Deployer
            try {
                MBeansHelper.getInstance().registerMBean(Deployer.getInstance());
            } catch (MBeansException e) {
                throw new EmbeddedException("Cannot init MBeans", e);
            }

            // register the Server
            try {
                MBeansHelper.getInstance().registerMBean(this);
            } catch (MBeansException e) {
                throw new EmbeddedException("Cannot init MBeans", e);
            }
}


        // Bind the TM factory
        if (config.startTM()) {
            JTransactionManager.getTransactionManager();
        }


        // Bind a dummy datasource
        if (config.bindDummyDataSourceHSQL()) {
            Server server = new Server();
            String baseDir = System.getProperty("java.io.tmpdir");
            String serverProps = "database.0=" + baseDir + File.separator + "jdbc_1" + ";dbname.0=jdbc_1;";
            serverProps += "database.1=" + baseDir + File.separator + "jdbc_2" + ";dbname.1=jdbc_2";
            server.putPropertiesFromString(serverProps);
            server.start();

            try {
                new InitialContext().rebind("jdbc_1", new DummyDataSource("jdbc_1"));
            } catch (NamingException e) {
                throw new EmbeddedException("Cannot start the dummy In-memory Datasource", e);
            }

            try {
                new InitialContext().rebind("jdbc_2", new DummyDataSource("jdbc_2"));
            } catch (NamingException e) {
                throw new EmbeddedException("Cannot start the dummy In-memory Datasource", e);
            }
        }

        // JMS ?
        if (config.startJMS()) {
            joramService = new JoramService();
            // Defines some dummy topics / queues
            joramService.setQueues(Arrays.asList(new String[] {"dummyQueue"}));
            joramService.setTopics(Arrays.asList(new String[] {"dummyTopic"}));
            try {
                joramService.start();
            } catch (JoramException e) {
                throw new EmbeddedException("Cannot start JORAM JMS provider", e);
            }
        }

        // Bind the RPC object
        RMIServerRPC invoker = null;
        try {
            invoker = new RMIServerRPCImpl(this);
        } catch (RemoteException e) {
            throw new EmbeddedException("Cannot build RPC invoker", e);
        }
        try {
            new InitialContext().rebind(RMIServerRPC.RPC_JNDI_NAME, invoker);
        } catch (NamingException e) {
            throw new EmbeddedException("Cannot bind the RPC invoker", e);
        }


        // add a thread which monitor containers file
        ContainersMonitor monitor = new ContainersMonitor(this);
        monitor.init();

        logger.info("Embedded.start.startup", Long.valueOf(System.currentTimeMillis() - tStart));
        logger.info("Embedded.start.created", Integer.valueOf(containers.size()));

        logger.info("Embedded.start.waiting");
        monitor.start();

        if (config.shouldWait()) {
            while (true) {
                try {
                    Thread.sleep(SLEEP_VALUE);
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
            }
        }
    }

    /**
     * Stops the EJB3 server.
     * @throws EmbeddedException if container cannot be stopped.
     */
    public void stop() throws EmbeddedException {

        // JMS provider was started ?
        if (config.startJMS()) {
            JoramService joramService = new JoramService();
            try {
                joramService.stop();
            } catch (JoramException e) {
                throw new EmbeddedException("Cannot stop Joram collocated server", e);
            }
        }

        // registry was started ?
        if (config.startRegistry()) {
            try {
                NameServiceManager.stopNS();
            } catch (NameServiceException e) {
                throw new EmbeddedException("Cannot stop the registry", e);
            }
        }
    }

    /**
     * Gets a container managed by this server.
     * @param id the container id.
     * @return the container if it is found, else null.
     */
    public EZBContainer getContainer(final String id) {
        return containers.get(id);
    }

    /**
     * Configure the server by using the given configuration.
     */
    private void configure() {
        File ejb3Directory = config.getEjb3Directory();
        // need to define a repository
        if (ejb3Directory == null) {
            ejb3Directory = new File("ejb3s");
            logger.warn("No directory was configured, take the default value of '" + ejb3Directory.getAbsolutePath() + "'.");
            if (!ejb3Directory.exists()) {
                if (ejb3Directory.mkdir()) {
                    logger.warn("Directory '" + ejb3Directory.getAbsolutePath() + "' created.");
                } else {
                    throw new IllegalStateException("Cannot make directory '" + ejb3Directory.getAbsolutePath() + "'.");
                }
            }
            // update the value
            config.setEjb3Path(ejb3Directory.getAbsolutePath());
        }
    }


    /**
     * Creates and adds an ejb3 container to the managed container.
     * @param archive jar file or exploded archive.
     * @return the created container.
     */
    public EZBContainer createContainer(final File archive) {
        EZBContainerConfig jConfig = new JContainerConfig(archive);
        jConfig.setEZBServer(this);
        callJContainerConfigExtensions(jConfig);
        EZBContainer container = new JContainer3(jConfig);
        String id = container.getId();
        containers.put(id, container);

        if (getConfig().isUsingMBeans()) {
            try {
                MBeansHelper.getInstance().registerMBean(container);
            } catch (MBeansException e) {
                // TODO what to do here ? log or exception ?
                logger.error("Cannot register Container MBeans for " + archive, e);
            }
        }

        return container;
    }

    /**
     * Adapt the JContainerConfig for all {@link EasyBeansConfigurationExtension}.
     * @param jcc the JContainerConfig to adapt.
     */
    private void callJContainerConfigExtensions(final EZBContainerConfig jcc) {

        for (EasyBeansConfigurationExtension extension : this.extensionFactories) {
            try {
                extension.configure(jcc);
            } catch (Throwable t) {
                // prevent malicious code to break everything ...
                logger.info("Failed to configure JContainerConfig with {0}",
                        extension.getClass().getName());
            }
        }
    }

    /**
     * Main method called by default.
     * @param args the arguments for the main method
     * @throws Exception if failures
     */
    public static void main(final String[] args) throws Exception {

        // Configure server
        ServerConfig config = new ServerConfig();
        Embedded embedded = new Embedded(config);

        // Start new embedded server
        embedded.start();
    }


    /**
     * @return the configuration of this server.
     */
    protected ServerConfig getConfig() {
        return config;
    }

    /**
     * @return the containers managed by this server.
     */
    public Map<String, EZBContainer> getContainers() {
        return containers;
    }

    /**
     * Starts the registry.
     * @throws EmbeddedException if registry can't be started
     */
    private void startRegistry() throws EmbeddedException{

        // Set carol mode in server mode (will use fixed port if set)
        System.setProperty("carol.server.mode", "true");
        try {
            // hack
            Writer fw;
            File fConf = new File(System.getProperty("java.io.tmpdir") + File.separator + "ejb3-carol.properties");
            try {
                fw = new FileWriter(fConf);
                fw.write("carol.protocols=jrmp\n");
                fw.write("carol.jrmp.url=rmi://localhost:1099\n");
                fw.write("carol.jvm.rmi.local.registry=true");
                fw.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
            ConfigurationRepository.init(fConf.toURL());
        } catch (ConfigurationException e) {
            throw new EmbeddedException("Cannot initialize registry", e);
        } catch (MalformedURLException e) {
            throw new EmbeddedException("Cannot initialize registry", e);
        }

        if (config.startTM()) {
            try {
                ConfigurationRepository.addInterceptors("jrmp", "org.objectweb.jotm.jta.rmi.JTAInterceptorInitializer");
            } catch (ConfigurationException e) {
                throw new EmbeddedException("Cannot add JOTM interceptors", e);
            }
        }

        try {
            NameServiceManager.startNS();
        } catch (NameServiceException e) {
            throw new EmbeddedException("Cannot start registry", e);
        }
    }

    /**
     * Gets the id of this embedded server.
     * @return the id of this server.
     */
    public Integer getID() {
        return id;
    }

    /**
     * Gets the joram service (null if not started)
     * @return joram service instance.
     */
    public JoramService getJoramService() {
        return joramService;
    }
}
