/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: InterceptorHelper.java 703 2006-06-21 14:33:25Z studzine $
 * --------------------------------------------------------------------------
 */
package org.objectweb.easybeans.tests.common.helper;

import java.util.ArrayList;
import java.util.List;

import javax.interceptor.InvocationContext;
import javax.jms.Message;

import org.objectweb.easybeans.log.JLog;
import org.objectweb.easybeans.log.JLogFactory;

/**
 * This helper is used to do operation with InvocationContext and interceptors.
 * @author Eduardo Studzinski Estima de Castro
 * @author Gisele Pinheiro Souza
 */
public final class InterceptorHelper {

    /**
     * Order Property.
     */
    public static final String PROPERTY_ORDER = "ORDER";

    /**
     * Helper should have a private constructor.
     */
    private InterceptorHelper() {

    }

    /**
     * Intercepts the method and adds a value in the list that was get from
     * InvocationContext.
     * @param <E> Element Type
     * @param invocationContext contains attributes of invocation, the first
     *        parameter of the intercepted method must be a list or a message.
     * @param value value to add
     * @param className class that is invoking this method.
     * @return method's invocation result
     * @throws Exception if invocation fails
     */
    @SuppressWarnings("unchecked")
    public static <E> Object addValue(final InvocationContext invocationContext, final E value, final String className)
            throws Exception {
        JLog logger = JLogFactory.getLog(InterceptorHelper.class);

        logger.debug("Adding: order={0}", value);

        Object[] arObj = invocationContext.getParameters();

        List<E> arOrder = null;

        if (arObj[0] instanceof List) {
            arOrder = (List<E>) arObj[0];

        }else if (arObj[0] instanceof Message){

            Message msg = (Message) arObj[0];

            //Checks if the property exists.
            if (msg.getObjectProperty(PROPERTY_ORDER) == null) {
                arOrder = new ArrayList<E>();
                msg.setObjectProperty(PROPERTY_ORDER, arOrder);
            }else{
                arOrder = (List<E>) msg.getObjectProperty(PROPERTY_ORDER);
            }
        }

        // If can't add, throw an exception to avoid cascade errors.
        if (!arOrder.add(value)) {
            throw new Exception(className + " can not add the value " + value + ".");
        }
        return invocationContext.proceed();
    }

    /**
     * Gets the error msg formatted.
     * @param expected values to a correct answer
     * @param result result values after an execution
     * @param seedMessage standard message to show
     * @return error message
     */
    public static String getPrintOrderErrorMsg(final List expected, final List result, final String seedMessage) {
        String strMessage = seedMessage;

        if (expected == null & result == null) {
            strMessage += " Result=[null], Expected=[null].";
        } else if (expected == null) {
            strMessage += " Result=" + result.toString() + ", Expected=[null].";
        } else if (result == null) {
            strMessage += " Result=[null], Expected=" + expected.toString() + ".";
        } else {
            strMessage += " Actual=" + result.toString() + ", Expected=" + expected.toString() + ".";
        }
        return strMessage;
    }
}
