/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: TestInvocation00.java 703 2006-06-21 14:33:25Z studzine $
 * --------------------------------------------------------------------------
 */
package org.objectweb.easybeans.tests.interceptors.invocationcontext;

import static org.objectweb.easybeans.tests.common.helper.EJBHelper.getBeanRemoteInstance;
import static org.objectweb.easybeans.tests.common.helper.InvocationContextHelper.INT_VALUE_0;
import static org.objectweb.easybeans.tests.common.helper.InvocationContextHelper.INT_VALUE_1;
import static org.objectweb.easybeans.tests.common.helper.InvocationContextHelper.INT_VALUE_2;
import static org.objectweb.easybeans.tests.common.helper.InvocationContextHelper.STR_VALUE_0;
import static org.objectweb.easybeans.tests.common.helper.InvocationContextHelper.STR_VALUE_1;
import static org.objectweb.easybeans.tests.common.helper.InvocationContextHelper.STR_VALUE_2;
import static org.testng.Assert.assertEquals;
import static org.testng.Assert.assertNull;
import static org.testng.Assert.assertTrue;

import java.util.List;

import org.objectweb.easybeans.log.JLog;
import org.objectweb.easybeans.log.JLogFactory;
import org.objectweb.easybeans.tests.common.ejbs.base.ItfInvocation00;
import org.objectweb.easybeans.tests.common.ejbs.base.ItfInvocationParameter00;
import org.objectweb.easybeans.tests.common.ejbs.base.ItfInvocationParameter01;
import org.objectweb.easybeans.tests.common.ejbs.base.ItfOneMethod00;
import org.objectweb.easybeans.tests.common.ejbs.stateless.containermanaged.SLSBInvocationParameterTest00;
import org.objectweb.easybeans.tests.common.ejbs.stateless.containermanaged.SLSBInvocationParameterTest01;
import org.objectweb.easybeans.tests.common.ejbs.stateless.containermanaged.SLSBInvocationTest00;
import org.objectweb.easybeans.tests.common.ejbs.stateless.containermanaged.SLSBInvocationTest01;
import org.objectweb.easybeans.tests.common.ejbs.stateless.containermanaged.SLSBInvocationTest02;
import org.objectweb.easybeans.tests.common.ejbs.stateless.containermanaged.SLSBInvocationTest03;
import org.objectweb.easybeans.tests.common.interceptors.invocationcontext.BeanDescriptor;
import org.objectweb.easybeans.tests.common.interceptors.invocationcontext.ComplexObject00;
import org.testng.annotations.Test;

/**
 * Verifies if the invocation context is following the JSR220 spec.
 * @reference JSR220-PROPOSED FINAL
 * @requirement Application Server must be running; the bean
 *              org.objectweb.easybeans.tests.common.ejbs.stateless.SLSB*Invocation*
 *              must be deployed.
 * @setup gets the reference of SLSBInvocation*
 * @author Eduardo Studzinski Estima de Castro
 * @author Gisele Pinheiro Souza
 */
public class TestInvocation00 {

    /**
     * Logger.
     */
    private JLog logger = JLogFactory.getLog(TestInvocation00.class);

    /**
     * Constant value.
     */
    public static final int VALUE = 5000;

    /**
     * Test message.
     */
    private final String strMsg00 = "Maybe the invocation context get/setParameters() aren't working well.";

    /**
     * Verifies if the InvocationContext in interceptor classes maintains the
     * same bean reference.
     * @input First parameter of the method must be a BeanDescriptor
     * @output Comparation Result the BeanDescriptor created by the Client and
     *         the BeanDescriptor created into the Interceptor Method.
     * @throws Exception if there is a problem with the test.
     */
    @Test(groups = {"invocationContext"})
    public void testInvocation00() throws Exception {
        testInvocation(SLSBInvocationTest00.class);
    }
    /**
     * Verifies if the InvocationContext in interceptor method of a bean class
     * maintains the same bean reference.
     * @input First parameter of the method must be a BeanDescriptor
     * @output Comparation Result the BeanDescriptor created by the Client and
     *         the BeanDescriptor created into the Interceptor Method.
     * @throws Exception if there is a problem with the test.
     */
    @Test(groups = {"invocationContext"})
    public void testInvocation01() throws Exception {
        testInvocation(SLSBInvocationTest01.class);
    }

    /**
     * Verifies if the InvocationContext in interceptor method of a bean class
     * maintains the same bean reference.
     * @param beanClass bean class
     * @throws Exception Exception if there is a problem with the test.
     */
    private void testInvocation(final Class beanClass) throws Exception {
        ItfInvocation00 icBean = getBeanRemoteInstance(beanClass, ItfInvocation00.class);

        // Creates the bean to compare
        Class[] types = {BeanDescriptor.class};

        BeanDescriptor bDesc = new BeanDescriptor(icBean.hashCode(), icBean.getClass().getMethod("getBool", types)
                .toString());

        assertTrue(icBean.getBool(bDesc), "This method must return true if the invocation context is running"
                + " following the specification. Maybe the bean is not referenced "
                + " correctly or the intercepted method is not the same that must be intercepted.");
    }

    /**
     * Verifies if the intercepted method parameters can be modified to a null
     * reference. The interceptor is an external class.
     * @input First parameter of the method must be a ComplexObject00
     * @input Second parameter of the method must be an array of Integers.
     * @output Comparation Result between the objects passed as parameters and
     *         the objects returned by the method.
     * @throws Exception if there is a problem with the test.
     */
    @Test(groups = {"invocationContext", "get/setParameters"})
    public void testParameters00() throws Exception {
        testNull(SLSBInvocationParameterTest00.class);
    }

    /**
     * Verifies if the intercepted method parameters can be modified to a null
     * reference. The interceptor method is on the bean class.
     * @input First parameter of the method must be a ComplexObject00
     * @input Second parameter of the method must be an array of Integers.
     * @output Comparation Result between the objects passed as parameters and
     *         the objects returned by the method.
     * @throws Exception if there is a problem with the test.
     */
    @Test(groups = {"invocationContext", "get/setParameters"})
    public void testParameters01() throws Exception {
        testNull(SLSBInvocationParameterTest01.class);
    }

    /**
     * Verifies if the intercepted method parameters can be modified to a null
     * reference.
     * @param beanClass bean class
     * @throws Exception if there is a problem with the test.
     */
    private void testNull(final Class beanClass) throws Exception {
        ItfInvocationParameter00 icBean = getBeanRemoteInstance(beanClass, ItfInvocationParameter00.class);

        ComplexObject00 cmpObj = new ComplexObject00();
        Integer[] arInts = new Integer[VALUE];

        Object[] arResult = icBean.getObjects00(cmpObj, arInts);

        assertNull(arResult[0], "[0]The object should be null. " + strMsg00);
        assertNull(arResult[1], "[1]The object should be null. " + strMsg00);
    }

    /**
     * Verifies if the intercepted method parameters can be returned WITHOUT
     * modifications. The interceptor is a external class.
     * @input First parameter of the method must be a ComplexObject00
     * @input Second parameter of the method must be an array of Integers.
     * @output Comparation Result between the objects passed as parameters and
     *         the objects returned by the method.
     * @throws Exception if there is a problem with the test.
     */
    @Test(groups = {"invocationContext", "get/setParameters"})
    public void testParameters02() throws Exception {
        testWithoutModification(SLSBInvocationParameterTest00.class);
    }

    /**
     * Verifies if the intercepted method parameters can be returned WITHOUT
     * modifications. The interceptor method is on the bean class.
     * @input First parameter of the method must be a ComplexObject00
     * @input Second parameter of the method must be an array of Integers.
     * @output Comparation Result between the objects passed as parameters and
     *         the objects returned by the method.
     * @throws Exception if there is a problem with the test.
     */
    @Test(groups = {"invocationContext", "get/setParameters"})
    public void testParameters03() throws Exception {
        testWithoutModification(SLSBInvocationParameterTest01.class);
    }

    /**
     * Verifies if the intercepted method parameters can be returned WITHOUT
     * modifications.
     * @param beanClass bean class name
     * @throws Exception if there is a problem with the test.
     */
    private void testWithoutModification(final Class beanClass) throws Exception{
        ItfInvocationParameter00 icBean = getBeanRemoteInstance(beanClass, ItfInvocationParameter00.class);

        ComplexObject00 cmpObj = new ComplexObject00();
        Integer[] arInts = new Integer[VALUE];

        logger.debug("before bean method call, parameters: {0}, {1}", cmpObj, arInts);

        Object[] arResult = icBean.getObjects01(cmpObj, arInts);

        logger.debug("after bean method call, parameters: {0}, {1}", arResult[0], arResult[1]);

        assertEquals(arResult[0], cmpObj, "[0]The object should be the same. " + strMsg00);
        assertEquals(arResult[1], arInts, "[1]The object should be the same. " + strMsg00);
    }

    /**
     * Verifies if the intercepted method parameters can be returned WITH
     * modifications. The interceptor is a external class.
     * @input First parameter of the method must be a ComplexObject00
     * @input Second parameter of the method must be an array of Integers.
     * @output Comparation Result between the objects passed as parameters and
     *         the objects returned by the method.
     * @throws Exception if there is a problem with the test.
     */
    @SuppressWarnings("boxing")
    @Test(groups = {"invocationContext", "get/setParameters"})
    public void testParameters04() throws Exception {
        testWithModification(SLSBInvocationParameterTest00.class);
    }

    /**
     * Verifies if the intercepted method parameters can be returned WITH
     * modifications. The interceptor method is on the bean class.
     * @input First parameter of the method must be a ComplexObject00
     * @input Second parameter of the method must be an array of Integers.
     * @output Comparation Result between the objects passed as parameters and
     *         the objects returned by the method.
     * @throws Exception if there is a problem with the test.
     */
    @SuppressWarnings("boxing")
    @Test(groups = {"invocationContext", "get/setParameters"})
    public void testParameters05() throws Exception {
        testWithModification(SLSBInvocationParameterTest01.class);
    }

    /**
     * Verifies if the intercepted method parameters can be returned WITH
     * modifications.
     * @param beanClass bean class
     * @throws Exception if there is a problem with the test.
     */
    private void testWithModification(final Class beanClass) throws Exception{
        ItfInvocationParameter00 icBean = getBeanRemoteInstance(beanClass, ItfInvocationParameter00.class);

        ComplexObject00 cmpObj = new ComplexObject00();
        Integer[] arInts = {INT_VALUE_0, INT_VALUE_1, INT_VALUE_2};

        Object[] arObjResult = icBean.getObjects02(cmpObj, arInts);

        ComplexObject00 cmpResult = (ComplexObject00) arObjResult[0];

        // Checks if the result is correct
        assertTrue(cmpResult.getHashCode() == INT_VALUE_0.intValue(), "[0]The returned object is not the expected. "
                + strMsg00);
        assertEquals(cmpResult.getInterceptedMethod(), STR_VALUE_0, "[1]The returned object is not the expected. "
                + strMsg00);

        List<BeanDescriptor> lstDescs = cmpResult.getDescriptors().get(0);
        // Checks the first bean descriptor
        assertTrue(lstDescs.get(0).getHashCode() == INT_VALUE_1.intValue(),
                "[2]The returned object is not the expected. " + strMsg00);
        assertEquals(lstDescs.get(0).getInterceptedMethod(), STR_VALUE_1,
                "[3]The returned object is not the expected. " + strMsg00);

        assertTrue(lstDescs.get(1).getHashCode() == INT_VALUE_2.intValue(),
                "[4]The returned object is not the expected. " + strMsg00);
        assertEquals(lstDescs.get(1).getInterceptedMethod(), STR_VALUE_2,
                "[5]The returned object is not the expected. " + strMsg00);

        // Checks array values
        Integer[] arIntResult = (Integer[]) arObjResult[1];
        assertTrue(arIntResult[0].equals(INT_VALUE_0), "[6]The returned object is not the expected. " + strMsg00);
        assertTrue(arIntResult[1].equals(INT_VALUE_1), "[7]The returned object is not the expected. " + strMsg00);
        assertTrue(arIntResult[2].equals(INT_VALUE_2), "[8]The returned object is not the expected. " + strMsg00);
    }

    /**
     * Verifies if the context data is working following the specification.
     * @throws Exception if there is a problem with the test.
     */
    @Test(groups = {"invocationContext", "getContextData"})
    public void testContextData00() throws Exception{
        ItfInvocationParameter01 icBean = getBeanRemoteInstance(SLSBInvocationTest02.class, ItfInvocationParameter01.class);
        icBean.getObjects(null, null, null);
    }

    /**
     * Verifies if the context data is working following the specification.
     * @throws Exception if there is a problem with the test.
     */
    @Test(groups = {"invocationContext", "getContextData"})
    @SuppressWarnings("boxing")
    public void testContextData01() throws Exception{
        ItfInvocationParameter01 icBean = getBeanRemoteInstance(SLSBInvocationTest02.class, ItfInvocationParameter01.class);
        icBean.getObjects(new ComplexObject00(), new Integer[INT_VALUE_2], new BeanDescriptor());
    }

    /**
     * Verifies if the context data is working following the specification.
     * @throws Exception if there is a problem with the test.
     */
    @Test(groups = {"invocationContext", "getContextData"})
    @SuppressWarnings("boxing")
    public void testContextData02() throws Exception{
        ItfInvocationParameter01 icBean = getBeanRemoteInstance(SLSBInvocationTest02.class, ItfInvocationParameter01.class);
        icBean.getObjects(new ComplexObject00(), null, new BeanDescriptor());
    }

    /**
     * Verifies if the proceed is returning null with a void interceptor method.
     * @throws Exception if there is a problem with the test.
     */
    @Test(groups = {"invocationContext", "proceed"})
    public void testProceed00() throws Exception{
        ItfOneMethod00 icBean = getBeanRemoteInstance(SLSBInvocationTest03.class, ItfOneMethod00.class);
        icBean.doOne(null);
    }
}
