/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: TestMandatoryExceptionRemoteSFSB.java 439 2006-05-10 11:21:09Z pinheirg $
 * --------------------------------------------------------------------------
 */
package org.objectweb.easybeans.tests.transaction.containermanaged.stateful;

import java.sql.SQLException;

import javax.ejb.EJBTransactionRequiredException;

import org.objectweb.easybeans.tests.common.ejbs.stateful.containermanaged.SFSBContainerTransactionMandatoryApp;
import org.objectweb.easybeans.tests.common.ejbs.stateful.containermanaged.SFSBContainerTransactionMandatoryRollback;
import org.objectweb.easybeans.tests.common.ejbs.stateful.containermanaged.SFSBContainerTransactionMandatoryRuntime;
import org.objectweb.easybeans.tests.transaction.containermanaged.base.TestContainerTransactionException;
import org.objectweb.easybeans.tests.transaction.containermanaged.base.TestMandatoryException;
import org.testng.annotations.Configuration;
import org.testng.annotations.ExpectedExceptions;
import org.testng.annotations.Test;


/**
 * Verifies if the container-managed transaction in the session bean is
 * following the JSR220. The container must handle the different types of
 * exception in the transaction context. The items covered in this test are:
 * 13.3
 * @reference JSR220-PROPOSED FINAL
 * @requirement Application Server must be running; the bean .
 *              SFSBContainerTransactionMandatoryApp,
 *              SFSBContainerTransactionMandatoryRollback and
 *              SFSBContainerTransactionMandatoryRuntime must be deployed.
 * @setup gets the reference of the bean and binds the databases specified in
 *        the EmbeddedTest.
 * @author Gisele Pinheiro Souza
 * @author Eduardo Studzinski Estima de Castro
 */
public class TestMandatoryExceptionRemoteSFSB extends TestMandatoryException{

    /**
     * Initiates a user transaction and call a bean that throws an application
     * exception. The container must not make a rollback.
     * @input -
     * @output the method execution without error.
     * @throws Exception if an error occurs.
     */
    @Test(groups = {"application exception tests"})
    public void testIncorInsertionUsingClientTransApp() throws Exception {
          super.testIncorInsertionUsingClientTransAndApp();
    }

    /**
     * Initiates a user transaction and call a bean that throws an application
     * exception with rollback = true. The container must make a rollback.
     * @input -
     * @output a SQLException when the test tries to select the table created by
     *         the bean.
     * @throws Exception if an error occurs.
     */
    @Test(groups = {"application rollback exception tests"})
    @ExpectedExceptions(value = SQLException.class)
    public void testIncorInsertionUsingClientTransRollback() throws Exception {
          super.testIncorInsertionUsingClientTrans(TestContainerTransactionException.APP_ROLLBACK_EXCEPTION);
    }

    /**
     * Initiates a user transaction and call a bean that throws a runtime
     * exception. The container must make a rollback(does not commit the table),
     * discard the bean and send an EJBException.
     * @input -
     * @output a SQLException when the test tries to select the table created by
     *         the bean.
     * @throws Exception if an error occurs.
     */
    @Test(groups = {"runtime exception tests"})
    @ExpectedExceptions(value = SQLException.class)
    public void testIncorInsertionUsingClientTransRuntime() throws Exception {
          super.testIncorInsertionUsingClientTrans(TestContainerTransactionException.RUNTIME_EXCEPTION);
    }

    /**
     * @see org.objectweb.easybeans.tests.transaction.containermanaged.base.TestMandatoryException
     * @throws if an error during the tests occurs.
     */
    @Test(groups = {"application exception tests"})
    @Override
    @ExpectedExceptions(value = EJBTransactionRequiredException.class)
    public void testNotUsingClientTransWithAppException() throws Exception {
         super.testNotUsingClientTransWithAppException();
    }


    /**
     * Cleans the transaction actives before each test.
     * @throws Exception if the transaction cannot be closed.
     */
    @Configuration(beforeTestMethod = true)
    @Override
    public void cleanTransaction() throws Exception {
         super.cleanTransaction();
    }


    /**
     * @see org.objectweb.easybeans.tests.transaction.containermanaged.base.TestMandatoryException
     * @throws if an error during the tests occurs.
     */
    @Test(groups = {"runtime exception tests"})
    @Override
    public void testCallOtherBeanNotSup() throws Exception {
        super.testCallOtherBeanNotSup();
    }

    /**
     * @see org.objectweb.easybeans.tests.transaction.containermanaged.base.TestMandatoryException
     * @throws if an error during the tests occurs.
     */
    @Test(groups = {"runtime exception tests"})
    @Override
    public void testCallOtherBeanReq() throws Exception {
         super.testCallOtherBeanReq();
    }

    /**
     * Creates before each method the bean used during the tests.
     * @throws Exception if an error during the lookup occurs.
     */
    @Configuration(beforeTestMethod = true,groups = "application exception tests")
    public void createBeanApp() throws Exception {
          super.createBeanApp(SFSBContainerTransactionMandatoryApp.class);
    }

    /**
     * Creates before each method the bean used during the tests.
     * @throws Exception if an error during the lookup occurs.
     */
    @Configuration(beforeTestMethod = true,groups = "application rollback exception tests")
    public void createBeanRollback() throws Exception {
          super.createBeanRollback(SFSBContainerTransactionMandatoryRollback.class);
    }

    /**
     * Creates before each method the bean used during the tests.
     * @throws Exception if an error during the lookup occurs.
     */
    @Configuration(beforeTestMethod = true,groups = "runtime exception tests")
    public void createBeanRuntime() throws Exception {
         super.createBeanRuntime(SFSBContainerTransactionMandatoryRuntime.class);
    }

    /**
     * @see org.objectweb.easybeans.tests.transaction.containermanaged.base.TestContainerTransactionException
     */
    @Configuration(beforeTestMethod = true)
    @Override
    public void deleteTable() {
         super.deleteTable();
    }

    /**
     * @see org.objectweb.easybeans.tests.transaction.containermanaged.base.TestContainerTransactionException
     */
    @Configuration(beforeTestClass = true)
    @Override
    public void setup() throws Exception {
         super.setup();
    }

}
